use async_trait::async_trait;
use log::warn;

use crate::{
    models::task::TaskDetails,
    regular_tasks::{TaskHandler, TaskRunContext},
    static_content::possession_type::possession_data,
    DResult,
};
use std::time;

use super::comms::broadcast_to_room;

#[derive(Clone)]
pub struct RechargeTaskHandler;
#[async_trait]
impl TaskHandler for RechargeTaskHandler {
    async fn do_task(&self, ctx: &mut TaskRunContext) -> DResult<Option<time::Duration>> {
        let item_ref = match &ctx.task.details {
            TaskDetails::ChargeItem { ref item } => item,
            t => {
                warn!("RechargeTaskHandler got unexpected task {:#?}", t);
                return Ok(None);
            }
        };
        let (item_type, item_code) = match item_ref.split_once("/") {
            Some(v) => v,
            None => {
                warn!("RechargeTaskHandler got unexpected item ref {}", item_ref);
                return Ok(None);
            }
        };
        let item = match ctx
            .trans
            .find_item_by_type_code(item_type, item_code)
            .await?
        {
            None => return Ok(None),
            Some(v) => v,
        };

        let charge_data = match item
            .possession_type
            .as_ref()
            .and_then(|pt| possession_data().get(pt))
            .and_then(|pd| pd.charge_data.as_ref())
        {
            None => return Ok(None),
            Some(v) => v,
        };
        let mut item = (*item).clone();

        item.charges = (item.charges + 1).min(charge_data.max_charges);
        ctx.trans.save_item_model(&item).await?;
        if item.charges == charge_data.max_charges {
            broadcast_to_room(
                &ctx.trans,
                &item.location,
                None,
                &format!(
                    "{} beeps to indicate it is fully charged.\n",
                    item.display_for_sentence(1, true)
                ),
            )
            .await?;
            Ok(None)
        } else {
            Ok(Some(time::Duration::from_secs(60)))
        }
    }
}
pub static TASK_HANDLER: &(dyn TaskHandler + Sync + Send) = &RechargeTaskHandler;
